package ch.hslu.appe.gui.model;

import java.math.BigDecimal;
import java.util.List;

import javax.swing.JOptionPane;
import javax.swing.table.AbstractTableModel;

import ch.hslu.appe.inventory.webservice.ProductDto;
import ch.hslu.appe.order.webservice.OrderDto;
import ch.hslu.appe.order.webservice.OrderPositionDto;
import ch.hslu.appe.order.webservice.SOAPException_Exception;
import ch.hslu.appe.webservice.consumer.InventoryConsumerImpl;
import ch.hslu.appe.webservice.consumer.OrderConsumerImpl;

/**
 * OrderTableModel.
 */
@SuppressWarnings("serial")
public class OrderTableModel extends AbstractTableModel {

    private final String[] tableHeader;
    private Object[][] tableData;
    private OrderDto activeOrder;

    /**
     * Constructor.
     * @param activeOrder activeOrder
     */
    public OrderTableModel(final OrderDto activeOrder) {

        this.activeOrder = activeOrder;
        tableHeader = new String[] {"Position", "Anzahl", "Produkt", "Preis",  "Preis Total"};

        generateTableData(this.activeOrder);
    }

    /**
     * Gets the column class.
     * @param c colum
     * @return column class
     */
    @SuppressWarnings({ "unchecked", "rawtypes" })
    @Override
    public final Class getColumnClass(final int c) {
        return getValueAt(0, c).getClass();
    }

    /**
     * Gets the number of columns.
     * @return number of columns
     */
    @Override
    public final int getColumnCount() {
        return tableHeader.length;
    }

    /**
     * Gets the name of the column.
     * @param col column
     * @return name of the column
     */
    @Override
    public final String getColumnName(final int col) {
        return tableHeader[col];
    }

    /**
     * Gets the number of rows.
     * @return number of rows
     */
    @Override
    public final int getRowCount() {
        return tableData.length;
    }

    /**
     * Gets the Table Entry.
     * @param row row
     * @param col col
     * @return entry
     */
    @Override
    public final Object getValueAt(final int row, final int col) {
        return tableData[row][col];
    }

    @Override
    public final void setValueAt(final Object value, final int row, final int col) {
        tableData[row][col] = value;
        fireTableCellUpdated(row, col);
    }

    /**
     * Generate / Update Table Data.
     * @param order order
     */
    public final void generateTableData(final OrderDto order) {

        if (order == null) {
            try {
                this.activeOrder = OrderConsumerImpl.getInstance().getActiveOrder();
            } catch (SOAPException_Exception e) {
                JOptionPane.showMessageDialog(null, "Es ist ein SOAP-Fehler aufgetreten!", "ERROR",
                        JOptionPane.ERROR_MESSAGE);
            }
        } else {
            this.activeOrder = order;
        }

        List<OrderPositionDto> orderPositions = this.activeOrder.getOrderPositions();

        if (orderPositions == null) {
            return;
        }

        tableData = new Object[orderPositions.size()][5];

        for (int index = 0; index < orderPositions.size(); index++) {
            OrderPositionDto orderPosition = orderPositions.get(index);
            ProductDto product = InventoryConsumerImpl.getInstance().getProduct(orderPosition.getProductNumber());

            tableData[index][0] = orderPosition.getPosition();
            tableData[index][1] = orderPosition.getAmount();
            tableData[index][2] = product.getProductname();
            tableData[index][3] = String.format("%.2f", round(orderPosition.getPrice(), 2));
            tableData[index][4] = String.format("%.2f", round(orderPosition.getPrice() * orderPosition.getAmount(), 2));
        }
    }

    /**
     * round.
     * @param number number
     * @param decimalPlace decimalPlace
     * @return formated float
     */
    private float round(final float number, final int decimalPlace) {
        BigDecimal bd = new BigDecimal(Float.toString(number));
        bd = bd.setScale(decimalPlace, BigDecimal.ROUND_HALF_UP);
        return bd.floatValue();
    }

}
