package ch.hslu.appe.webservice.consumer;

import java.util.ArrayList;

import ch.hslu.appe.usermanagement.webservice.AddressDto;
import ch.hslu.appe.usermanagement.webservice.IllegalNameException_Exception;
import ch.hslu.appe.usermanagement.webservice.UserDto;

/**
 * UserManagementConsumer.
 */
public interface UserManagementConsumer {

    /**
     * addAddress.
     * @param userId the user id
     * @param address address
     * @return successfully
     */
    boolean addAddress(int userId, AddressDto address);

    /**
     * addUser.
     * @param user user
     * @return successfully
     * @throws IllegalNameException_Exception IllegalNameException_Exception
     */
    boolean addUser(UserDto user) throws IllegalNameException_Exception;

    /**
     * deleteAddress.
     * @param addressId addressId
     * @return successfully
     */
    boolean deleteAddress(int addressId);

    /**
     * deleteUser.
     * @param userId userId
     * @return successfully
     */
    boolean deleteUser(int userId);

    /**
     * getAddress.
     * @param addressId addressId
     * @return AddressDto
     */
    AddressDto getAddress(int addressId);

    /**
     * getAddressesById.
     * @param userId userId
     * @return ArrayList of AddressDTOs
     */
    ArrayList<AddressDto> getAddressesById(int userId);

    /**
     * getAddressesByLogin.
     * @param login login
     * @return ArrayList of AddressDTOs
     */
    ArrayList<AddressDto> getAddressesByLogin(String login);

    /**
     * getUserById.
     * @param userId userId
     * @return UserDTO
     */
    UserDto getUserById(int userId);

    /**
     * getUserByLogin.
     * @param login login
     * @return UserDto
     */
    UserDto getUserByLogin(String login);

    /**
     * getUsers.
     * @return ArrayList of AddressDTOs
     */
    ArrayList<UserDto> getUsers();

    /**
     * updateAddress.
     * @param address address
     * @return successfully
     */
    boolean updateAddress(AddressDto address);

    /**
     * updateUser.
     * @param user user
     * @return successfully
     * @throws IllegalNameException_Exception IllegalNameException_Exception
     */
    boolean updateUser(UserDto user) throws IllegalNameException_Exception;
}
